package ch.psi.daq.queryrest.controller.validator;

import java.util.LinkedHashSet;
import java.util.Set;

import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.validation.Errors;
import org.springframework.validation.Validator;

import ch.psi.daq.domain.backend.Backend;
import ch.psi.daq.domain.config.DomainConfig;
import ch.psi.daq.domain.query.DAQConfigQuery;
import ch.psi.daq.domain.query.operation.ConfigField;
import ch.psi.daq.queryrest.config.QueryRestConfig;

public class ConfigQueryValidator implements Validator, ApplicationContextAware {
   private Set<ConfigField> queryResponseFields;

   @SuppressWarnings("unchecked")
   @Override
   public void setApplicationContext(ApplicationContext context) throws BeansException {
      final Backend backend = context.getBean(DomainConfig.BEAN_NAME_BACKEND_DEFAULT, Backend.class);
      context = backend.getApplicationContext();

      queryResponseFields = context.getBean(QueryRestConfig.BEAN_NAME_CONFIG_RESPONSE_FIELDS_QUERY, Set.class);
   }

   @Override
   public boolean supports(final Class<?> clazz) {
      return DAQConfigQuery.class.isAssignableFrom(clazz);
   }

   @Override
   public void validate(final Object target, final Errors errors) {
      if (target instanceof DAQConfigQuery) {
         this.checkElement((DAQConfigQuery) target, errors);
      }
   }

   private void checkElement(final DAQConfigQuery query, final Errors errors) {
      // set default values (if not set)
      if (query.getFields() == null || query.getFields().isEmpty()) {
         query.setFields(new LinkedHashSet<>(queryResponseFields));
      }
   }
}
